/* valaccodeswitchstatement.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <ccode/valaccodeswitchstatement.h>
#include <stdlib.h>
#include <string.h>
#include <ccode/valaccodenode.h>
#include <ccode/valaccodelinedirective.h>




struct _ValaCCodeSwitchStatementPrivate {
	ValaCCodeExpression* _expression;
};

#define VALA_CCODE_SWITCH_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_SWITCH_STATEMENT, ValaCCodeSwitchStatementPrivate))
enum  {
	VALA_CCODE_SWITCH_STATEMENT_DUMMY_PROPERTY
};
static void vala_ccode_switch_statement_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static gpointer vala_ccode_switch_statement_parent_class = NULL;
static void vala_ccode_switch_statement_finalize (ValaCCodeNode* obj);



ValaCCodeSwitchStatement* vala_ccode_switch_statement_construct (GType object_type, ValaCCodeExpression* expression) {
	ValaCCodeSwitchStatement* self;
	g_return_val_if_fail (expression != NULL, NULL);
	self = (ValaCCodeSwitchStatement*) g_type_create_instance (object_type);
	vala_ccode_switch_statement_set_expression (self, expression);
	return self;
}


ValaCCodeSwitchStatement* vala_ccode_switch_statement_new (ValaCCodeExpression* expression) {
	return vala_ccode_switch_statement_construct (VALA_TYPE_CCODE_SWITCH_STATEMENT, expression);
}


static void vala_ccode_switch_statement_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeSwitchStatement * self;
	self = (ValaCCodeSwitchStatement*) base;
	g_return_if_fail (writer != NULL);
	vala_ccode_writer_write_indent (writer, vala_ccode_node_get_line ((ValaCCodeNode*) self));
	vala_ccode_writer_write_string (writer, "switch (");
	vala_ccode_node_write ((ValaCCodeNode*) self->priv->_expression, writer);
	vala_ccode_writer_write_string (writer, ")");
	VALA_CCODE_NODE_CLASS (vala_ccode_switch_statement_parent_class)->write ((ValaCCodeNode*) VALA_CCODE_BLOCK (self), writer);
}


ValaCCodeExpression* vala_ccode_switch_statement_get_expression (ValaCCodeSwitchStatement* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_expression;
}


void vala_ccode_switch_statement_set_expression (ValaCCodeSwitchStatement* self, ValaCCodeExpression* value) {
	ValaCCodeExpression* _tmp2;
	ValaCCodeExpression* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_expression = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_ccode_node_ref (_tmp1)), (self->priv->_expression == NULL) ? NULL : (self->priv->_expression = (vala_ccode_node_unref (self->priv->_expression), NULL)), _tmp2);
}


static void vala_ccode_switch_statement_class_init (ValaCCodeSwitchStatementClass * klass) {
	vala_ccode_switch_statement_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_NODE_CLASS (klass)->finalize = vala_ccode_switch_statement_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeSwitchStatementPrivate));
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_switch_statement_real_write;
}


static void vala_ccode_switch_statement_instance_init (ValaCCodeSwitchStatement * self) {
	self->priv = VALA_CCODE_SWITCH_STATEMENT_GET_PRIVATE (self);
}


static void vala_ccode_switch_statement_finalize (ValaCCodeNode* obj) {
	ValaCCodeSwitchStatement * self;
	self = VALA_CCODE_SWITCH_STATEMENT (obj);
	(self->priv->_expression == NULL) ? NULL : (self->priv->_expression = (vala_ccode_node_unref (self->priv->_expression), NULL));
	VALA_CCODE_NODE_CLASS (vala_ccode_switch_statement_parent_class)->finalize (obj);
}


GType vala_ccode_switch_statement_get_type (void) {
	static GType vala_ccode_switch_statement_type_id = 0;
	if (vala_ccode_switch_statement_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeSwitchStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_switch_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeSwitchStatement), 0, (GInstanceInitFunc) vala_ccode_switch_statement_instance_init, NULL };
		vala_ccode_switch_statement_type_id = g_type_register_static (VALA_TYPE_CCODE_BLOCK, "ValaCCodeSwitchStatement", &g_define_type_info, 0);
	}
	return vala_ccode_switch_statement_type_id;
}




