/* valaccodedeclarationspace.vala
 *
 * Copyright (C) 2009  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <gobject/valaccodedeclarationspace.h>
#include <gee/hashset.h>
#include <gee/set.h>
#include <gee/collection.h>
#include <ccode/valaccodeincludedirective.h>
#include <gobject/gvaluecollector.h>




struct _ValaCCodeDeclarationSpacePrivate {
	GeeSet* includes;
};

#define VALA_CCODE_DECLARATION_SPACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpacePrivate))
enum  {
	VALA_CCODE_DECLARATION_SPACE_DUMMY_PROPERTY
};
static gpointer vala_ccode_declaration_space_parent_class = NULL;
static void vala_ccode_declaration_space_finalize (ValaCCodeDeclarationSpace* obj);



void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	if (!gee_collection_contains ((GeeCollection*) self->priv->includes, filename)) {
		ValaCCodeIncludeDirective* _tmp0;
		_tmp0 = NULL;
		vala_ccode_fragment_append (self->include_directives, (ValaCCodeNode*) (_tmp0 = vala_ccode_include_directive_new (filename, local)));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (vala_ccode_node_unref (_tmp0), NULL));
		gee_collection_add ((GeeCollection*) self->priv->includes, filename);
	}
}


void vala_ccode_declaration_space_add_begin (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	vala_ccode_fragment_append (self->begin, node);
}


void vala_ccode_declaration_space_add_type_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	vala_ccode_fragment_append (self->type_declaration, node);
}


void vala_ccode_declaration_space_add_type_definition (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	vala_ccode_fragment_append (self->type_definition, node);
}


void vala_ccode_declaration_space_add_type_member_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	vala_ccode_fragment_append (self->type_member_declaration, node);
}


void vala_ccode_declaration_space_add_constant_declaration (ValaCCodeDeclarationSpace* self, ValaCCodeNode* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	vala_ccode_fragment_append (self->constant_declaration, node);
}


ValaCCodeDeclarationSpace* vala_ccode_declaration_space_construct (GType object_type) {
	ValaCCodeDeclarationSpace* self;
	self = (ValaCCodeDeclarationSpace*) g_type_create_instance (object_type);
	return self;
}


ValaCCodeDeclarationSpace* vala_ccode_declaration_space_new (void) {
	return vala_ccode_declaration_space_construct (VALA_TYPE_CCODE_DECLARATION_SPACE);
}


static void vala_value_ccode_declaration_space_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_ccode_declaration_space_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_ccode_declaration_space_unref (value->data[0].v_pointer);
	}
}


static void vala_value_ccode_declaration_space_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_ccode_declaration_space_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_ccode_declaration_space_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_ccode_declaration_space_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaCCodeDeclarationSpace* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_ccode_declaration_space_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_ccode_declaration_space_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaCCodeDeclarationSpace** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_ccode_declaration_space_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecCCodeDeclarationSpace* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_CCODE_DECLARATION_SPACE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_ccode_declaration_space (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_DECLARATION_SPACE), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object) {
	ValaCCodeDeclarationSpace* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_CCODE_DECLARATION_SPACE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_CCODE_DECLARATION_SPACE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_ccode_declaration_space_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_ccode_declaration_space_unref (old);
	}
}


static void vala_ccode_declaration_space_class_init (ValaCCodeDeclarationSpaceClass * klass) {
	vala_ccode_declaration_space_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_DECLARATION_SPACE_CLASS (klass)->finalize = vala_ccode_declaration_space_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeDeclarationSpacePrivate));
}


static void vala_ccode_declaration_space_instance_init (ValaCCodeDeclarationSpace * self) {
	self->priv = VALA_CCODE_DECLARATION_SPACE_GET_PRIVATE (self);
	self->priv->includes = (GeeSet*) gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_str_hash, g_str_equal);
	self->begin = vala_ccode_fragment_new ();
	self->include_directives = vala_ccode_fragment_new ();
	self->type_declaration = vala_ccode_fragment_new ();
	self->type_definition = vala_ccode_fragment_new ();
	self->type_member_declaration = vala_ccode_fragment_new ();
	self->constant_declaration = vala_ccode_fragment_new ();
	self->ref_count = 1;
}


static void vala_ccode_declaration_space_finalize (ValaCCodeDeclarationSpace* obj) {
	ValaCCodeDeclarationSpace * self;
	self = VALA_CCODE_DECLARATION_SPACE (obj);
	(self->priv->includes == NULL) ? NULL : (self->priv->includes = (gee_collection_object_unref (self->priv->includes), NULL));
	(self->begin == NULL) ? NULL : (self->begin = (vala_ccode_node_unref (self->begin), NULL));
	(self->include_directives == NULL) ? NULL : (self->include_directives = (vala_ccode_node_unref (self->include_directives), NULL));
	(self->type_declaration == NULL) ? NULL : (self->type_declaration = (vala_ccode_node_unref (self->type_declaration), NULL));
	(self->type_definition == NULL) ? NULL : (self->type_definition = (vala_ccode_node_unref (self->type_definition), NULL));
	(self->type_member_declaration == NULL) ? NULL : (self->type_member_declaration = (vala_ccode_node_unref (self->type_member_declaration), NULL));
	(self->constant_declaration == NULL) ? NULL : (self->constant_declaration = (vala_ccode_node_unref (self->constant_declaration), NULL));
}


GType vala_ccode_declaration_space_get_type (void) {
	static GType vala_ccode_declaration_space_type_id = 0;
	if (vala_ccode_declaration_space_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_ccode_declaration_space_init, vala_value_ccode_declaration_space_free_value, vala_value_ccode_declaration_space_copy_value, vala_value_ccode_declaration_space_peek_pointer, "p", vala_value_ccode_declaration_space_collect_value, "p", vala_value_ccode_declaration_space_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeDeclarationSpaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_declaration_space_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeDeclarationSpace), 0, (GInstanceInitFunc) vala_ccode_declaration_space_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_ccode_declaration_space_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaCCodeDeclarationSpace", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_ccode_declaration_space_type_id;
}


gpointer vala_ccode_declaration_space_ref (gpointer instance) {
	ValaCCodeDeclarationSpace* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_ccode_declaration_space_unref (gpointer instance) {
	ValaCCodeDeclarationSpace* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_CCODE_DECLARATION_SPACE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




