/* valamember.vala
 *
 * Copyright (C) 2006-2008  Raffaele Sandrini, Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <vala/valamember.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/readonlylist.h>
#include <vala/valasourcereference.h>
#include <vala/valaclass.h>
#include <vala/valascope.h>
#include <vala/valastruct.h>
#include <vala/valacodevisitor.h>
#include <vala/valasourcefile.h>




struct _ValaMemberPrivate {
	GeeList* cheader_filenames;
	gboolean _hides;
};

#define VALA_MEMBER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_MEMBER, ValaMemberPrivate))
enum  {
	VALA_MEMBER_DUMMY_PROPERTY
};
static void vala_member_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static GeeList* vala_member_real_get_cheader_filenames (ValaSymbol* base);
static gpointer vala_member_parent_class = NULL;
static void vala_member_finalize (ValaCodeNode* obj);



ValaMember* vala_member_construct (GType object_type, const char* name, ValaSourceReference* source_reference) {
	ValaMember* self;
	self = (ValaMember*) vala_symbol_construct (object_type, name, source_reference);
	return self;
}


ValaMember* vala_member_new (const char* name, ValaSourceReference* source_reference) {
	return vala_member_construct (VALA_TYPE_MEMBER, name, source_reference);
}


static void vala_member_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
	ValaMember * self;
	self = (ValaMember*) base;
	g_return_if_fail (visitor != NULL);
	vala_code_visitor_visit_member (visitor, self);
}


static GeeList* vala_member_real_get_cheader_filenames (ValaSymbol* base) {
	ValaMember * self;
	gboolean _tmp0;
	self = (ValaMember*) base;
	_tmp0 = FALSE;
	if (gee_collection_get_size ((GeeCollection*) self->priv->cheader_filenames) == 0) {
		_tmp0 = vala_symbol_get_parent_symbol ((ValaSymbol*) self) != NULL;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		gboolean _tmp3;
		gboolean _tmp4;
		/* default to header filenames of the namespace */
		{
			GeeList* _tmp1;
			GeeIterator* _tmp2;
			GeeIterator* _filename_it;
			/* default to header filenames of the namespace */
			_tmp1 = NULL;
			_tmp2 = NULL;
			_filename_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vala_symbol_get_cheader_filenames (vala_symbol_get_parent_symbol ((ValaSymbol*) self)))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
			/* default to header filenames of the namespace */
			while (gee_iterator_next (_filename_it)) {
				char* filename;
				/* default to header filenames of the namespace */
				filename = (char*) gee_iterator_get (_filename_it);
				vala_member_add_cheader_filename (self, filename);
				filename = (g_free (filename), NULL);
			}
			(_filename_it == NULL) ? NULL : (_filename_it = (gee_collection_object_unref (_filename_it), NULL));
		}
		_tmp3 = FALSE;
		_tmp4 = FALSE;
		if (gee_collection_get_size ((GeeCollection*) self->priv->cheader_filenames) == 0) {
			_tmp4 = vala_code_node_get_source_reference ((ValaCodeNode*) self) != NULL;
		} else {
			_tmp4 = FALSE;
		}
		if (_tmp4) {
			_tmp3 = !vala_symbol_get_external_package ((ValaSymbol*) self);
		} else {
			_tmp3 = FALSE;
		}
		if (_tmp3) {
			char* _tmp5;
			/* don't add default include directives for VAPI files*/
			_tmp5 = NULL;
			gee_collection_add ((GeeCollection*) self->priv->cheader_filenames, _tmp5 = vala_source_file_get_cinclude_filename (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) self))));
			_tmp5 = (g_free (_tmp5), NULL);
		}
	}
	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->cheader_filenames);
}


/**
 * Adds a filename to the list of C header filenames users of this data
 * type must include.
 *
 * @param filename a C header filename
 */
void vala_member_add_cheader_filename (ValaMember* self, const char* filename) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	gee_collection_add ((GeeCollection*) self->priv->cheader_filenames, filename);
}


ValaSymbol* vala_member_get_hidden_member (ValaMember* self) {
	ValaSymbol* sym;
	ValaSymbol* _tmp12;
	g_return_val_if_fail (self != NULL, NULL);
	sym = NULL;
	if (VALA_IS_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) self))) {
		ValaClass* _tmp0;
		ValaClass* cl;
		_tmp0 = NULL;
		cl = (_tmp0 = vala_class_get_base_class (VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) self))), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
		while (cl != NULL) {
			ValaSymbol* _tmp1;
			gboolean _tmp2;
			ValaClass* _tmp5;
			ValaClass* _tmp4;
			_tmp1 = NULL;
			sym = (_tmp1 = vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) cl), vala_symbol_get_name ((ValaSymbol*) self)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp1);
			_tmp2 = FALSE;
			if (sym != NULL) {
				_tmp2 = vala_symbol_get_access (sym) != VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
			} else {
				_tmp2 = FALSE;
			}
			if (_tmp2) {
				ValaSymbol* _tmp3;
				_tmp3 = NULL;
				return (_tmp3 = sym, (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), _tmp3);
			}
			_tmp5 = NULL;
			_tmp4 = NULL;
			cl = (_tmp5 = (_tmp4 = vala_class_get_base_class (cl), (_tmp4 == NULL) ? NULL : vala_code_node_ref (_tmp4)), (cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL)), _tmp5);
		}
		(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
	} else {
		if (VALA_IS_STRUCT (vala_symbol_get_parent_symbol ((ValaSymbol*) self))) {
			ValaStruct* _tmp6;
			ValaStruct* st;
			_tmp6 = NULL;
			st = (_tmp6 = vala_struct_get_base_struct (VALA_STRUCT (vala_symbol_get_parent_symbol ((ValaSymbol*) self))), (_tmp6 == NULL) ? NULL : vala_code_node_ref (_tmp6));
			while (st != NULL) {
				ValaSymbol* _tmp7;
				gboolean _tmp8;
				ValaStruct* _tmp11;
				ValaStruct* _tmp10;
				_tmp7 = NULL;
				sym = (_tmp7 = vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) st), vala_symbol_get_name ((ValaSymbol*) self)), (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp7);
				_tmp8 = FALSE;
				if (sym != NULL) {
					_tmp8 = vala_symbol_get_access (sym) != VALA_SYMBOL_ACCESSIBILITY_PRIVATE;
				} else {
					_tmp8 = FALSE;
				}
				if (_tmp8) {
					ValaSymbol* _tmp9;
					_tmp9 = NULL;
					return (_tmp9 = sym, (st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL)), _tmp9);
				}
				_tmp11 = NULL;
				_tmp10 = NULL;
				st = (_tmp11 = (_tmp10 = vala_struct_get_base_struct (st), (_tmp10 == NULL) ? NULL : vala_code_node_ref (_tmp10)), (st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL)), _tmp11);
			}
			(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
		}
	}
	_tmp12 = NULL;
	return (_tmp12 = NULL, (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp12);
}


gboolean vala_member_get_hides (ValaMember* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_hides;
}


void vala_member_set_hides (ValaMember* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_hides = value;
}


static void vala_member_class_init (ValaMemberClass * klass) {
	vala_member_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_NODE_CLASS (klass)->finalize = vala_member_finalize;
	g_type_class_add_private (klass, sizeof (ValaMemberPrivate));
	VALA_CODE_NODE_CLASS (klass)->accept = vala_member_real_accept;
	VALA_SYMBOL_CLASS (klass)->get_cheader_filenames = vala_member_real_get_cheader_filenames;
}


static void vala_member_instance_init (ValaMember * self) {
	self->priv = VALA_MEMBER_GET_PRIVATE (self);
	self->priv->cheader_filenames = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
}


static void vala_member_finalize (ValaCodeNode* obj) {
	ValaMember * self;
	self = VALA_MEMBER (obj);
	(self->priv->cheader_filenames == NULL) ? NULL : (self->priv->cheader_filenames = (gee_collection_object_unref (self->priv->cheader_filenames), NULL));
	VALA_CODE_NODE_CLASS (vala_member_parent_class)->finalize (obj);
}


GType vala_member_get_type (void) {
	static GType vala_member_type_id = 0;
	if (vala_member_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaMemberClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_member_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaMember), 0, (GInstanceInitFunc) vala_member_instance_init, NULL };
		vala_member_type_id = g_type_register_static (VALA_TYPE_SYMBOL, "ValaMember", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vala_member_type_id;
}



GType member_binding_get_type (void) {
	static GType member_binding_type_id = 0;
	if (G_UNLIKELY (member_binding_type_id == 0)) {
		static const GEnumValue values[] = {{MEMBER_BINDING_INSTANCE, "MEMBER_BINDING_INSTANCE", "instance"}, {MEMBER_BINDING_CLASS, "MEMBER_BINDING_CLASS", "class"}, {MEMBER_BINDING_STATIC, "MEMBER_BINDING_STATIC", "static"}, {0, NULL, NULL}};
		member_binding_type_id = g_enum_register_static ("MemberBinding", values);
	}
	return member_binding_type_id;
}




