/* valaunresolvedtype.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <vala/valaunresolvedtype.h>
#include <gee/list.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valacodenode.h>
#include <vala/valamemberaccess.h>
#include <vala/valareport.h>




struct _ValaUnresolvedTypePrivate {
	ValaUnresolvedSymbol* _unresolved_symbol;
};

#define VALA_UNRESOLVED_TYPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_UNRESOLVED_TYPE, ValaUnresolvedTypePrivate))
enum  {
	VALA_UNRESOLVED_TYPE_DUMMY_PROPERTY
};
static ValaDataType* vala_unresolved_type_real_copy (ValaDataType* base);
static char* vala_unresolved_type_real_to_qualified_string (ValaDataType* base, ValaScope* scope);
static gpointer vala_unresolved_type_parent_class = NULL;
static void vala_unresolved_type_finalize (ValaCodeNode* obj);



ValaUnresolvedType* vala_unresolved_type_construct (GType object_type) {
	ValaUnresolvedType* self;
	self = (ValaUnresolvedType*) g_type_create_instance (object_type);
	return self;
}


ValaUnresolvedType* vala_unresolved_type_new (void) {
	return vala_unresolved_type_construct (VALA_TYPE_UNRESOLVED_TYPE);
}


/**
 * Creates a new type reference.
 *
 * @param symbol    unresolved type symbol
 * @param source    reference to source code
 * @return          newly created type reference
 */
ValaUnresolvedType* vala_unresolved_type_construct_from_symbol (GType object_type, ValaUnresolvedSymbol* symbol, ValaSourceReference* source) {
	ValaUnresolvedType* self;
	g_return_val_if_fail (symbol != NULL, NULL);
	self = (ValaUnresolvedType*) g_type_create_instance (object_type);
	vala_unresolved_type_set_unresolved_symbol (self, symbol);
	vala_code_node_set_source_reference ((ValaCodeNode*) self, source);
	return self;
}


ValaUnresolvedType* vala_unresolved_type_new_from_symbol (ValaUnresolvedSymbol* symbol, ValaSourceReference* source) {
	return vala_unresolved_type_construct_from_symbol (VALA_TYPE_UNRESOLVED_TYPE, symbol, source);
}


/**
 * Creates a new type reference from a code expression.
 *
 * @param expr   member access expression
 * @param source reference to source code
 * @return       newly created type reference
 */
ValaUnresolvedType* vala_unresolved_type_new_from_expression (ValaExpression* expr) {
	g_return_val_if_fail (expr != NULL, NULL);
	if (VALA_IS_MEMBER_ACCESS (expr)) {
		ValaUnresolvedType* type_ref;
		ValaMemberAccess* _tmp0;
		ValaMemberAccess* ma;
		type_ref = NULL;
		_tmp0 = NULL;
		ma = (_tmp0 = VALA_MEMBER_ACCESS (expr), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
		if (vala_member_access_get_inner (ma) != NULL) {
			if (VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (ma))) {
				ValaMemberAccess* _tmp1;
				ValaMemberAccess* simple;
				ValaUnresolvedType* _tmp4;
				ValaUnresolvedSymbol* _tmp3;
				ValaUnresolvedSymbol* _tmp2;
				_tmp1 = NULL;
				simple = (_tmp1 = VALA_MEMBER_ACCESS (vala_member_access_get_inner (ma)), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
				_tmp4 = NULL;
				_tmp3 = NULL;
				_tmp2 = NULL;
				type_ref = (_tmp4 = vala_unresolved_type_new_from_symbol (_tmp3 = vala_unresolved_symbol_new (_tmp2 = vala_unresolved_symbol_new (NULL, vala_member_access_get_member_name (simple), vala_code_node_get_source_reference ((ValaCodeNode*) ma)), vala_member_access_get_member_name (ma), vala_code_node_get_source_reference ((ValaCodeNode*) ma)), vala_code_node_get_source_reference ((ValaCodeNode*) ma)), (type_ref == NULL) ? NULL : (type_ref = (vala_code_node_unref (type_ref), NULL)), _tmp4);
				(_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL));
				(_tmp2 == NULL) ? NULL : (_tmp2 = (vala_code_node_unref (_tmp2), NULL));
				(simple == NULL) ? NULL : (simple = (vala_code_node_unref (simple), NULL));
			}
		} else {
			ValaUnresolvedType* _tmp6;
			ValaUnresolvedSymbol* _tmp5;
			_tmp6 = NULL;
			_tmp5 = NULL;
			type_ref = (_tmp6 = vala_unresolved_type_new_from_symbol (_tmp5 = vala_unresolved_symbol_new (NULL, vala_member_access_get_member_name (ma), vala_code_node_get_source_reference ((ValaCodeNode*) ma)), vala_code_node_get_source_reference ((ValaCodeNode*) ma)), (type_ref == NULL) ? NULL : (type_ref = (vala_code_node_unref (type_ref), NULL)), _tmp6);
			(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_code_node_unref (_tmp5), NULL));
		}
		if (type_ref != NULL) {
			GeeList* type_args;
			ValaUnresolvedType* _tmp7;
			vala_data_type_set_value_owned ((ValaDataType*) type_ref, TRUE);
			type_args = vala_member_access_get_type_arguments (ma);
			{
				GeeIterator* _arg_it;
				_arg_it = gee_iterable_iterator ((GeeIterable*) type_args);
				while (gee_iterator_next (_arg_it)) {
					ValaDataType* arg;
					arg = (ValaDataType*) gee_iterator_get (_arg_it);
					vala_data_type_add_type_argument ((ValaDataType*) type_ref, arg);
					(arg == NULL) ? NULL : (arg = (vala_code_node_unref (arg), NULL));
				}
				(_arg_it == NULL) ? NULL : (_arg_it = (gee_collection_object_unref (_arg_it), NULL));
			}
			_tmp7 = NULL;
			return (_tmp7 = type_ref, (type_args == NULL) ? NULL : (type_args = (gee_collection_object_unref (type_args), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), _tmp7);
		}
		(type_ref == NULL) ? NULL : (type_ref = (vala_code_node_unref (type_ref), NULL));
		(ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL));
	}
	vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) expr), "Type reference must be simple name or member access expression");
	return NULL;
}


static ValaDataType* vala_unresolved_type_real_copy (ValaDataType* base) {
	ValaUnresolvedType * self;
	ValaUnresolvedType* result;
	ValaUnresolvedSymbol* _tmp0;
	self = (ValaUnresolvedType*) base;
	result = vala_unresolved_type_new ();
	vala_code_node_set_source_reference ((ValaCodeNode*) result, vala_code_node_get_source_reference ((ValaCodeNode*) self));
	vala_data_type_set_value_owned ((ValaDataType*) result, vala_data_type_get_value_owned ((ValaDataType*) self));
	vala_data_type_set_nullable ((ValaDataType*) result, vala_data_type_get_nullable ((ValaDataType*) self));
	vala_data_type_set_is_dynamic ((ValaDataType*) result, vala_data_type_get_is_dynamic ((ValaDataType*) self));
	_tmp0 = NULL;
	vala_unresolved_type_set_unresolved_symbol (result, _tmp0 = vala_unresolved_symbol_copy (self->priv->_unresolved_symbol));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (vala_code_node_unref (_tmp0), NULL));
	{
		GeeList* _tmp1;
		GeeIterator* _tmp2;
		GeeIterator* _arg_it;
		_tmp1 = NULL;
		_tmp2 = NULL;
		_arg_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vala_data_type_get_type_arguments ((ValaDataType*) self))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
		while (gee_iterator_next (_arg_it)) {
			ValaDataType* arg;
			ValaDataType* _tmp3;
			arg = (ValaDataType*) gee_iterator_get (_arg_it);
			_tmp3 = NULL;
			vala_data_type_add_type_argument ((ValaDataType*) result, _tmp3 = vala_data_type_copy (arg));
			(_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL));
			(arg == NULL) ? NULL : (arg = (vala_code_node_unref (arg), NULL));
		}
		(_arg_it == NULL) ? NULL : (_arg_it = (gee_collection_object_unref (_arg_it), NULL));
	}
	return (ValaDataType*) result;
}


static char* vala_unresolved_type_real_to_qualified_string (ValaDataType* base, ValaScope* scope) {
	ValaUnresolvedType * self;
	self = (ValaUnresolvedType*) base;
	return vala_code_node_to_string ((ValaCodeNode*) self->priv->_unresolved_symbol);
}


ValaUnresolvedSymbol* vala_unresolved_type_get_unresolved_symbol (ValaUnresolvedType* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_unresolved_symbol;
}


void vala_unresolved_type_set_unresolved_symbol (ValaUnresolvedType* self, ValaUnresolvedSymbol* value) {
	ValaUnresolvedSymbol* _tmp2;
	ValaUnresolvedSymbol* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_unresolved_symbol = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_unresolved_symbol == NULL) ? NULL : (self->priv->_unresolved_symbol = (vala_code_node_unref (self->priv->_unresolved_symbol), NULL)), _tmp2);
}


static void vala_unresolved_type_class_init (ValaUnresolvedTypeClass * klass) {
	vala_unresolved_type_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_NODE_CLASS (klass)->finalize = vala_unresolved_type_finalize;
	g_type_class_add_private (klass, sizeof (ValaUnresolvedTypePrivate));
	VALA_DATA_TYPE_CLASS (klass)->copy = vala_unresolved_type_real_copy;
	VALA_DATA_TYPE_CLASS (klass)->to_qualified_string = vala_unresolved_type_real_to_qualified_string;
}


static void vala_unresolved_type_instance_init (ValaUnresolvedType * self) {
	self->priv = VALA_UNRESOLVED_TYPE_GET_PRIVATE (self);
}


static void vala_unresolved_type_finalize (ValaCodeNode* obj) {
	ValaUnresolvedType * self;
	self = VALA_UNRESOLVED_TYPE (obj);
	(self->priv->_unresolved_symbol == NULL) ? NULL : (self->priv->_unresolved_symbol = (vala_code_node_unref (self->priv->_unresolved_symbol), NULL));
	VALA_CODE_NODE_CLASS (vala_unresolved_type_parent_class)->finalize (obj);
}


GType vala_unresolved_type_get_type (void) {
	static GType vala_unresolved_type_type_id = 0;
	if (vala_unresolved_type_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaUnresolvedTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_unresolved_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaUnresolvedType), 0, (GInstanceInitFunc) vala_unresolved_type_instance_init, NULL };
		vala_unresolved_type_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaUnresolvedType", &g_define_type_info, 0);
	}
	return vala_unresolved_type_type_id;
}




